"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (babel) {
  var t = babel.types;

  /**
   * t.isFunctionExpression() || t.isArrowFunctionExpression()
   */

  function isAnyFunctionExpression() {
    return t.isFunctionExpression.apply(t, arguments) || t.isArrowFunctionExpression.apply(t, arguments);
  }

  function isAction(node, actionIdentifier, mobxNamespaceIdentifier) {
    return actionIdentifier && t.isIdentifier(node, { name: actionIdentifier }) || mobxNamespaceIdentifier && t.isMemberExpression(node) && t.isIdentifier(node.object, { name: mobxNamespaceIdentifier }) && t.isIdentifier(node.property, { name: "action" });
  }

  var traverseActionBody = _defineProperty({}, "FunctionExpression|ArrowFunctionExpression", function FunctionExpressionArrowFunctionExpression(path) {
    var actionIdentifier = this.actionIdentifier;
    var mobxNamespaceIdentifier = this.mobxNamespaceIdentifier;
    path.get('body').traverse(traverseActionBody, { actionIdentifier: actionIdentifier, mobxNamespaceIdentifier: mobxNamespaceIdentifier });
    path.skip();
    // if current node parent is call expression and this call is action call, skip wrapping
    if (t.isCallExpression(path.parentPath.node) && isAction(path.parentPath.node.callee, actionIdentifier, mobxNamespaceIdentifier)) {
      return;
    }
    path.replaceWith(t.CallExpression(this.actionIdentifier ? t.Identifier(this.actionIdentifier) : t.MemberExpression(t.Identifier(this.mobxNamespaceIdentifier), t.Identifier("action")), [path.node]));
  });

  var traverseSibling = _defineProperty({
    CallExpression: function CallExpression(path) {
      var node = path.node;
      var actionIdentifier = this.actionIdentifier;
      var mobxNamespaceIdentifier = this.mobxNamespaceIdentifier;
      if (isAction(node.callee, actionIdentifier, mobxNamespaceIdentifier)) {
        if (node.arguments.length === 1) {
          path.get('arguments.0').traverse(traverseActionBody, { actionIdentifier: actionIdentifier, mobxNamespaceIdentifier: mobxNamespaceIdentifier });
          path.skip();
        } else if (node.arguments.length === 2) {
          path.get('arguments.1').traverse(traverseActionBody, { actionIdentifier: actionIdentifier, mobxNamespaceIdentifier: mobxNamespaceIdentifier });
          path.skip();
        }
      }
    }
  }, "ClassMethod|ClassProperty", function ClassMethodClassProperty(path) {
    var actionIdentifier = this.actionIdentifier;
    var mobxNamespaceIdentifier = this.mobxNamespaceIdentifier;
    if (path.node.decorators) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = path.node.decorators[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var _ref2 = _step.value;
          var expression = _ref2.expression;

          if (isAction(expression, actionIdentifier, mobxNamespaceIdentifier) || t.isCallExpression(expression) && isAction(expression.callee, actionIdentifier, mobxNamespaceIdentifier)) {
            if (t.isClassMethod(path.node)) {
              path.get('body').traverse(traverseActionBody, { actionIdentifier: actionIdentifier, mobxNamespaceIdentifier: mobxNamespaceIdentifier });
              path.skip();
            } else if (t.isClassProperty(path.node)) {
              path.get('value').traverse(traverseActionBody, { actionIdentifier: actionIdentifier, mobxNamespaceIdentifier: mobxNamespaceIdentifier });
              path.skip();
            }
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
  });

  return {
    name: "mobx-make-action-deep-transform", // not required
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        if (path.node.source.value === "mobx") {
          var _iteratorNormalCompletion2 = true;
          var _didIteratorError2 = false;
          var _iteratorError2 = undefined;

          try {
            for (var _iterator2 = path.node.specifiers[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
              var specifier = _step2.value;

              var actionIdentifier = void 0;
              var mobxNamespaceIdentifier = void 0;
              if (t.isImportNamespaceSpecifier(specifier) || specifier.imported.name === "action") {
                if (t.isImportNamespaceSpecifier(specifier)) {
                  mobxNamespaceIdentifier = specifier.local.name;
                } else if (specifier.imported.name === "action") {
                  actionIdentifier = specifier.local.name;
                }
                for (var index = path.key + 1; index < path.container.length; index++) {
                  path.getSibling(index).traverse(traverseSibling, { actionIdentifier: actionIdentifier, mobxNamespaceIdentifier: mobxNamespaceIdentifier });
                }
              }
            }
          } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
              }
            } finally {
              if (_didIteratorError2) {
                throw _iteratorError2;
              }
            }
          }
        }
      }
    }
  };
};

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }